#include "hvif-light.h"

#include <stdint.h>
#include <stdlib.h>

char const* const eof_error = "File ended prematurely.";

#define le_read(V, BUFFER) V = _Generic((V), uint32_t : le_read_uint32)(BUFFER)
static inline uint32_t
le_read_uint32(char buffer[static 1])
{
	return (buffer[0] << 0) | (buffer[1] << 8) | (buffer[2] << 16) |
		(buffer[3] << 24);
}

#define ERROR_RESULT(M) \
	(hvif_result) { .success = false, .error = M }

struct hvif_image
{};

hvif_result
hvif_from_file(FILE* file)
{
	uint32_t const magic = 0x6669636e; /* 'finc' (on disk little endian 'cnif') */

	char magic_buffer[4];
	if (fread(magic_buffer, 1, 4, file) != 4) { return ERROR_RESULT(eof_error); }
	uint32_t read_magic;
	le_read(read_magic, magic_buffer);
	if (read_magic != magic) {
		printf("%04x %04x\n", magic, read_magic);
		return ERROR_RESULT("Wrong magic number.");
	}

	hvif_image* image = malloc(sizeof(hvif_image));
	return (hvif_result){ .success = true, .image = image };
}

void
hvif_free(hvif_image* image)
{
	free(image);
}
